package com.ruoyi.ss.payBill.service.impl;

import com.ruoyi.ss.channel.domain.ChannelVO;
import com.ruoyi.ss.channel.service.ChannelService;
import com.ruoyi.ss.payBill.domain.PayBill;
import com.ruoyi.ss.payBill.domain.PayBillVO;
import com.ruoyi.ss.payBill.domain.enums.PayBillBstType;
import com.ruoyi.ss.payBill.domain.enums.PayBillStatus;
import com.ruoyi.ss.payBill.service.PayBillConverter;
import com.ruoyi.ss.timeBill.domain.TimeBill;
import com.ruoyi.ss.transactionBill.domain.bo.RechargePayDepositBO;
import com.ruoyi.ss.transactionBill.domain.dto.RechargePayBO;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillPayType;
import com.ruoyi.ss.transactionBill.domain.vo.TransactionBillVO;
import com.ruoyi.ss.user.domain.SmUserVo;
import com.ruoyi.ss.user.service.ISmUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;

/**
 * @author wjh
 * 2024/8/9
 */
@Service
public class PayBillConverterImpl implements PayBillConverter {

    @Autowired
    private ISmUserService userService;

    @Autowired
    private ChannelService channelService;

    @Override
    public PayBill toPo(TimeBill bill, Long channelId) {
        if (bill == null) {
            return null;
        }

        PayBill po = new PayBill();

        // 查询用户
        SmUserVo user = userService.selectSmUserByUserId(bill.getUserId());
        if (user == null) {
            return null;
        }

        // 查询渠道
        ChannelVO channel = channelService.selectSmChannelByChannelId(channelId);

        // 若关闭成功，创建支付订单
        PayBillVO data = new PayBillVO();
        data.setBstType(PayBillBstType.TIME_BILL.getType());
        data.setBstId(bill.getBillId());
        data.setAmount(bill.getAmount());
        data.setChannelId(channelId);
        data.setStatus(PayBillStatus.WAIT_PAY.getStatus());
        data.setDescription("计时订单" + bill.getBillNo());

        // 渠道成本
        data.setChannelCost(this.calcChannelCost(channel, bill.getAmount()));

        // 支付人
        if (TransactionBillPayType.WECHAT.getType().equals(data.getChannelId())) {
            data.setAccount(user.getWxOpenId());
        }
        return po;
    }

    /**
     * 计算渠道成本
     */
    private BigDecimal calcChannelCost(ChannelVO channel, BigDecimal amount) {
        if (channel == null || amount == null) {
            return null;
        }
        return channel.getCostRate().multiply(amount).divide(new BigDecimal(100), 2, RoundingMode.HALF_UP);
    }

    @Override
    public PayBill toPoByRechargeMoney(RechargePayBO bo) {
        if (bo == null) {
            return null;
        }

        TransactionBillVO order = bo.getOrder();
        if (order == null) {
            return null;
        }

        ChannelVO channel = bo.getChannel();
        if (channel == null) {
            return null;
        }

        SmUserVo user = bo.getUser();
        if (user == null) {
            return null;
        }

        PayBill po = new PayBill();
        po.setBstType(PayBillBstType.RECHARGE_ORDER.getType());
        po.setBstId(order.getBillId());
        po.setAmount(order.getMoney());
        po.setChannelId(channel.getChannelId());
        po.setStatus(PayBillStatus.WAIT_PAY.getStatus());
        po.setDescription("充值订单:" + order.getBillNo());
        po.setChannelCost(this.calcChannelCost(channel, order.getMoney()));

        // 支付人
        if (TransactionBillPayType.wxList().contains(channel.getChannelId())) {
            po.setAccount(user.getWxOpenId());
        }

        return po;
    }

    @Override
    public PayBill toPoByRechargeDeposit(RechargePayDepositBO bo) {

        if (bo == null) {
            return null;
        }

        TransactionBillVO order = bo.getOrder();
        if (order == null) {
            return null;
        }

        ChannelVO channel = bo.getChannel();
        if (channel == null) {
            return null;
        }

        SmUserVo user = bo.getUser();
        if (user == null) {
            return null;
        }

        PayBill po = new PayBill();
        po.setBstType(PayBillBstType.RECHARGE_ORDER_DEPOSIT.getType());
        po.setBstId(order.getBillId());
        po.setAmount(order.getSuitDeposit());
        po.setChannelId(channel.getChannelId());
        po.setStatus(PayBillStatus.WAIT_PAY.getStatus());
        po.setDescription("充值订单押金:" + order.getBillNo());
        po.setChannelCost(this.calcChannelCost(channel, order.getSuitPrice()));

        // 支付人
        if (TransactionBillPayType.wxList().contains(channel.getChannelId())) {
            po.setAccount(user.getWxOpenId());
        }

        return po;
    }
}
