package com.ruoyi.web.controller.app;

import com.fasterxml.jackson.annotation.JsonView;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.JsonViewProfile;
import com.ruoyi.common.core.domain.ValidGroup;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.enums.OperatorType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.iot.service.IotService;
import com.ruoyi.ss.device.domain.DeviceView;
import com.ruoyi.ss.device.domain.Device;
import com.ruoyi.ss.device.domain.DeviceBO;
import com.ruoyi.ss.device.domain.DeviceQuery;
import com.ruoyi.ss.device.domain.dto.DeviceRegisterDTO;
import com.ruoyi.ss.device.domain.dto.DeviceWifiDTO;
import com.ruoyi.ss.device.domain.enums.DevicePowerStatus;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.device.service.DeviceAssembler;
import com.ruoyi.ss.device.service.DeviceValidator;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.meterReadingRecord.domain.SmMeterReadingRecordQuery;
import com.ruoyi.ss.meterReadingRecord.service.ISmMeterReadingRecordService;
import com.ruoyi.ss.suit.domain.enums.SuitTimeUnit;
import com.ruoyi.web.core.annotation.DeviceAdminRequired;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 设备Controller
 * @author 辉
 * 2024/3/5
 */
@Api(tags = "设备")
@RestController
@RequestMapping("/app/device")
public class AppDeviceController extends BaseController {

    @Autowired
    private DeviceService smDeviceService;

    @Autowired
    private ISmMeterReadingRecordService smMeterReadingRecordService;

    @Autowired
    private DeviceAssembler deviceAssembler;

    @Autowired
    private DeviceValidator deviceValidator;

    @Autowired
    private IotService iotService;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Log(title = "商户修改设备信息", businessType = BusinessType.UPDATE, operatorType = OperatorType.MOBILE)
    @ApiOperation("商户修改设备信息")
    @PutMapping
    public AjaxResult edit(@RequestBody @Validated({ValidGroup.FrontUpdate.class}) DeviceBO data) {
        data = data.filterUpdateByApp();
        ServiceUtil.assertion(deviceValidator.preUpdateByApp(data));
        return toAjax(smDeviceService.updateSmDevice(data));
    }

    @ApiOperation("查询商户设备列表")
    @GetMapping("/list")
    public TableDataInfo list(DeviceQuery smDevice) {
        startPage();
        startOrderBy();
        smDevice.setUserId(getUserId());
        List<DeviceVO> list = smDeviceService.selectSmDeviceList(smDevice);
        smDeviceService.pullDeviceInfoList(list);
        deviceAssembler.assembleRealServiceRate(list);
        return getDataTable(list);
    }

    @ApiOperation("查询店铺设备列表")
    @GetMapping("/listByStore/{storeId}")
    @JsonView(JsonViewProfile.App.class)
    public TableDataInfo listByStore(@PathVariable("storeId") Long storeId, DeviceQuery query) {
        startPage();
        query.setStoreId(storeId);
        List<DeviceVO> list = smDeviceService.selectSmDeviceList(query);
        smDeviceService.pullDeviceInfoList(list);
        deviceAssembler.assembleBusinessTime(list); // 店铺营业时间
        return getDataTable(list);
    }

    @ApiOperation("获取设备详细信息")
    @GetMapping(value = "/{deviceId}")
    public AjaxResult getInfo(@PathVariable("deviceId") Long deviceId) {
//        smDeviceService.pullDeviceInfo(Collections.singletonList(deviceId));
        DeviceVO device = smDeviceService.selectById(deviceId);
        List<DeviceVO> list = Collections.singletonList(device);
        deviceAssembler.assembleOrderCountInfo(list);   // 订单统计信息
        return success(device);
    }

    @Log(title = "设备通断电", businessType = BusinessType.OTHER, operatorType = OperatorType.MOBILE)
    @ApiOperation("设备通断电")
    @PutMapping("/{deviceId}/changePower")
    public AjaxResult close(@PathVariable @ApiParam("设备id") Long deviceId, String status) {
        DevicePowerStatus powerStatus = DevicePowerStatus.parse(status);
        return success(smDeviceService.changePowerStatus(deviceId, powerStatus ));
    }

    @Log(title = "绑定设备", businessType = BusinessType.OTHER, operatorType = OperatorType.MOBILE)
    @ApiOperation("绑定设备")
    @PutMapping("/bind")
    public AjaxResult bind(@RequestBody Device device) {
        if (StringUtils.isAllBlank(device.getDeviceNo(), device.getMac())) {
            return error("设备编号和mac不能同时为空");
        }
        if (StringUtils.hasText(device.getDeviceNo())) {
            return AjaxResult.success("操作成功", smDeviceService.bindBySn(device.getStoreId(), getUserId(), device.getDeviceNo()));
        } else {
            return AjaxResult.success("操作成功", smDeviceService.bindByMac(device.getStoreId(), getUserId(), device.getMac()));
        }
    }

    @Log(title = "解除设备绑定", businessType = BusinessType.OTHER, operatorType = OperatorType.MOBILE)
    @ApiOperation("解除设备绑定")
    @DeleteMapping("/mch/unbind/{deviceId}")
    public AjaxResult mchUnbind(@PathVariable @ApiParam("设备id") Long deviceId) {
        ServiceUtil.assertion(!deviceValidator.isBelong(deviceId, getUserId()), "这不是您的设备");
        return toAjax(smDeviceService.unbind(deviceId));
    }

    @Log(title = "设备归零", businessType = BusinessType.OTHER, operatorType = OperatorType.MOBILE)
    @ApiOperation("设备归零")
    @PutMapping("{deviceId}/reset")
    public AjaxResult reset(@PathVariable @ApiParam("设备id") Long deviceId) {
        return toAjax(smDeviceService.resetWithBill(deviceId));
    }

    @ApiOperation("获取设备用电量分析")
    @GetMapping("/electric/count")
    public AjaxResult count(@Validated({ValidGroup.Query.class}) SmMeterReadingRecordQuery dto)
    {
        return success(smMeterReadingRecordService.selectCount(dto));
    }

    @Log(title = "商户充值设备", businessType = BusinessType.OTHER, operatorType = OperatorType.MOBILE)
    @ApiOperation("商户充值设备")
    @PutMapping("/addTime/{deviceId}")
    public AjaxResult addTime(@PathVariable @ApiParam("设备id") Long deviceId,
                              @ApiParam("数值") @RequestParam BigDecimal amount,
                              @ApiParam("单位") @RequestParam(required = false, defaultValue = "3") String timeUnit,
                              @ApiParam("是否操作物联网设备") @RequestParam(required = false, defaultValue = "true") Boolean withIot
    ) {
        ServiceUtil.assertion(!deviceValidator.isBelong(deviceId, getUserId()), "这不是您的设备");
        DeviceVO device = smDeviceService.selectById(deviceId);
        ServiceUtil.assertion(device == null || !getUserId().equals(device.getUserId()), "设备不存在或您无权充值");

        // 电量
        if ( "0".equals(timeUnit)) {
            return toAjax(smDeviceService.addEle(deviceId, amount, withIot, "商户手动充值"));
        }
        // 时长
        else {
            SuitTimeUnit unit = SuitTimeUnit.getByValue(timeUnit);
            if (unit == null) {
                return error("非法的时长单位");
            }
            long seconds = amount.multiply(new BigDecimal(unit.getConversion())).longValue();
            return toAjax(smDeviceService.addTimeByUser(deviceId, seconds, withIot, "商户手动充值"));
        }
    }

    @ApiOperation("刷新设备信息")
    @GetMapping("/{deviceId}/refreshIot")
    public AjaxResult syncIot(@PathVariable @ApiParam("设备id") Long deviceId) {
        smDeviceService.pullDeviceInfo(Collections.singletonList(deviceId));
        return success();
    }

    @ApiOperation("判断设备是否已经被绑定")
    @GetMapping("/isBind")
    public AjaxResult isBind(@RequestParam(required = false) @ApiParam("设备mac") String mac,
                              @RequestParam(required = false) @ApiParam("设备sn") String deviceNo) {
        if (StringUtils.hasText(mac)) {
            return success(smDeviceService.bindCodeByMac(mac));
        } else if (StringUtils.hasText(deviceNo)) {
            return success(smDeviceService.bindCodeBySn(deviceNo));
        } else {
            return error("至少传入一个MAC号或SN码");
        }
    }

    @ApiOperation("获取设备，包含套餐列表")
    @JsonView(DeviceView.SuitList.class)
    @GetMapping("/{deviceNo}/withSuitList")
    public AjaxResult getDeviceWithSuitList(@PathVariable String deviceNo) {
        return success(smDeviceService.getDeviceWithSuitList(deviceNo));
    }

    @ApiOperation("通过SN码获取设备信息")
    @GetMapping("/{deviceNo}/bySn")
    @JsonView(JsonViewProfile.App.class)
    public AjaxResult getByDeviceNo(@PathVariable @ApiParam("设备编号(SN)") String deviceNo) {
        return success(smDeviceService.selectByDeviceNo(deviceNo));
    }

    @Log(title = "管理员设备录入", businessType = BusinessType.INSERT, operatorType = OperatorType.MOBILE)
    @ApiOperation("管理员设备录入")
    @PostMapping
    @DeviceAdminRequired
    public AjaxResult add(@RequestBody @Validated DeviceRegisterDTO dto) {
        return toAjax(smDeviceService.register(dto));
    }

    @ApiOperation("管理员获取已录入的设备Mac列表")
    @GetMapping("/listAllMac")
    @DeviceAdminRequired
    public AjaxResult getAllMac() {
        return success(smDeviceService.selectMacSnList(new DeviceQuery()));
    }

    @ApiOperation("管理员过滤已录入的设备MAC和SN列表")
    @DeviceAdminRequired
    @GetMapping("/getExistMac/{mac}")
    public AjaxResult getExistMac(@PathVariable @ApiParam("设备mac") List<String> mac) {
        DeviceQuery query = new DeviceQuery();
        query.setMacList(mac);
        return success(smDeviceService.selectMacSnList(query));
    }

    @ApiOperation("管理员开关设备")
    @DeviceAdminRequired
    @PutMapping("/admin/{deviceId}/switch")
    public AjaxResult getExistMac(@PathVariable @ApiParam("设备ID") Long deviceId, @RequestParam @ApiParam("是否开启") Boolean open) {
        return toAjax(smDeviceService.switchDevice(deviceId, open));
    }

    @ApiOperation("获取正在使用中的设备列表")
    @GetMapping("/usingDevice")
    @JsonView(JsonViewProfile.App.class)
    public AjaxResult getUsingDevice() {
       return success(smDeviceService.selectUsingDevice(getUserId()));
    }

    @ApiOperation("设置设备WIFI密码")
    @PutMapping("/setWifi")
    public AjaxResult setWifi(@RequestBody DeviceWifiDTO dto) {

        return toAjax(smDeviceService.setWifi(dto));
    }

}
